local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Glyphs tested:
-- Everlasting Affliction
-- Dark Soul
-- Life Tap
-- Burning Embers
-- Conflagrate
-- Verdant Spheres

local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	61316--[[Dalaran Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
	126309--[[Still Water]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
	34889--[[Fire Breath]],
	24844--[[Lightning Breath]],
}

local StaminaBuff = {
	469--[[Commanding Shout]],
	21562--[[Power Word: Fortitude]],
	90364--[[Qiraji Fortitude]],
	109773--[[Dark Intent]],
}

a.spells["Pet Attack"] = {
	Override = function(z)
		if x.NoCC and x.PetAlive and x.Enemy and not UnitGUID("pettarget") then
			s.FlashPet(PET_ACTION_ATTACK, "Green")
		elseif x.NoCC and x.PetAlive and x.Enemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK, "Pink")
		end
	end,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and not a.PetAssistActive() then
			s.FlashPet(PET_MODE_ASSIST, "Pink")
		end
	end,
}

a.spells["dark_intent,if=!aura.spell_power_multiplier.up"] = {
	ID = 109773--[[Dark Intent]],
	NotWhileMoving = false,
	NoRangeCheck = true,
	FlashColor = "Green",
	Check = function(z)
		return not s.Buff(SpellPowerMultiplier, "raid") or not s.Buff(StaminaBuff, "raid")
	end,
}

a.spells["curse_of_the_elements,if=debuff.magic_vulnerability.down"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.NoCC and not x.Metamorphosis
	end,
}

a.spells["summon_pet,if=!talent.grimoire_of_sacrifice.enabled|buff.grimoire_of_sacrifice.down"] = {
	ID = 10--[[Summon Demon]],
	NotWhileMoving = true,
	Check = function(z)
		if x.Spec == 2 then
			z.FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]], 10--[[Summon Demon]]}
		else
			z.FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]], 10--[[Summon Demon]]}
		end
		return x.ShouldPermanentBuff and not x.PetAlive and (not s.HasTalent(108503--[[Grimoire of Sacrifice]]) or not s.Buff({132612, 132613, 132614--[[Grimoire of Sacrifice]]}, "player"))
	end,
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	MyBuff = 113860--[[Dark Soul: Misery]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		local charges, maxCharges = GetSpellCharges(113860--[[Dark Soul: Misery]])
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and charges == maxCharges and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	MyBuff = 113861--[[Dark Soul: Knowledge]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		local charges, maxCharges = GetSpellCharges(113861--[[Dark Soul: Knowledge]])
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and charges == maxCharges and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["Dark Soul: Instability"] = {
	ID = 113858--[[Dark Soul: Instability]],
	MyBuff = 113858--[[Dark Soul: Instability]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		local charges, maxCharges = GetSpellCharges(113861--[[Dark Soul: Knowledge]])
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and charges == maxCharges and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["service_pet,if=talent.grimoire_of_service.enabled"] = {
	ID = 111898--[[Grimoire: Felguard]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111859--[[Grimoire: Imp]],
			   111895--[[Grimoire: Voidwalker]],
			   111896--[[Grimoire: Succubus]],
			   111897--[[Grimoire: Felhunter]],
			   111898--[[Grimoire: Felguard]],
			   108501--[[Grimoire of Service]]},
}

a.spells["grimoire_of_sacrifice,if=talent.grimoire_of_sacrifice.enabled"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	MyBuff = {132612, 132613, 132614--[[Grimoire of Sacrifice]]},
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["summon_doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.Boss("target")
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
	FlashID = {18540--[[Summon Doomguard]], 112927--[[Summon Terrorguard]]}
}

a.spells["soul_swap,cycle_targets=1,if=buff.soulburn.up"] = {
	ID = 86121--[[Soul Swap]],
	ConsumeBuff = 74434--[[Soulburn]],
	ConsumeBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["soulburn,if=(buff.dark_soul.up|trinket.proc.intellect.react|trinket.stacking_proc.intellect.react>7)&(dot.agony.ticks_remain<=action.agony.add_ticks%2|dot.corruption.ticks_remain<=action.corruption.add_ticks%2|dot.unstable_affliction.ticks_remain<=action.unstable_affliction.add_ticks%2)&shard_react"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and GetSpellBonusDamage(6) > max(a.getDoTStat(980--[[Agony]], "spell_power"), a.getDoTStat(30108--[[Unstable Affliction]], "spell_power")) and (x.DarkSoulRemains > x.GCD or x.IntProc > x.GCD or x.TrinketIntProcStack > 7) and (x.AgonyTicksRemain * 2 <= x.AgonyTicks or x.CorruptionTicksRemain * 2 <= x.CorruptionTicks or x.UATicksRemain * 2 <= x.UATicks)
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["soulburn,if=(dot.unstable_affliction.ticks_remain<=1|dot.corruption.ticks_remain<=1|dot.agony.ticks_remain<=1)&shard_react&target.health.pct<=20"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and (x.UATicksRemain <= 1 or x.CorruptionTicksRemain <= 1 or x.AgonyTicksRemain <= 1) and s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["soulburn,if=stat.spell_power>dot.unstable_affliction.spell_power&dot.unstable_affliction.ticks_remain<=action.unstable_affliction.add_ticks%2&shard_react&target.health.pct<=20"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and GetSpellBonusDamage(6) > a.getDoTStat(30108--[[Unstable Affliction]], "spell_power") and x.UATicksRemain * 2 <= x.UATicks and s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["haunt,if=!in_flight_to_target&remains<cast_time+travel_time+tick_time&shard_react&target.health.pct<=20"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
	NotWhileMoving = true,
	Check = function(z)
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 1) + x.TickSpeed + s.CastTime(48181--[[Haunt]])
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
}

a.spells["haunt,if=!in_flight_to_target&remains<cast_time+travel_time+tick_time&(soul_shard>2|cooldown.dark_soul.remains>35|(soul_shard>1&cooldown.dark_soul.remains<cast_time))&shard_react"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
	NotWhileMoving = true,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 1) + x.TickSpeed + s.CastTime(48181--[[Haunt]])
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and (x.SoulShards > 2 or x.DarkSoulCD > 35 or (x.SoulShards > 1 and x.DarkSoulCD < s.CastTime(48181--[[Haunt]])))
	end,
	EnemyTargetNeeded = true,
}

a.spells["agony,cycle_targets=1,if=remains<gcd&remains+2<cooldown.dark_soul.remains&miss_react"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
	-- remains<gcd&remains+2<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and x.AgonyRemains < (x.MaxGCD + x.GCD) and x.AgonyRemains + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["agony,cycle_targets=1,if=ticks_remain<=2&remains+2<cooldown.dark_soul.remains&miss_react"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and (x.AgonyTicksRemain <= 2 or x.AgonyRemains < s.CastTime(103103--[[Malefic Grasp]]) + x.GCD) and x.AgonyRemains + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["agony,cycle_targets=1,if=stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains+2<cooldown.dark_soul.remains&miss_react"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(980--[[Agony]], "spell_power") and (x.TrinketIntProcStack == 0 or x.TrinketIntProcStack > 7) and x.AgonyTicksRemain * 2 < x.AgonyTicks and x.AgonyRemains + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=remains<gcd&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
	-- remains<gcd&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = min((x.MaxGCD + x.GCD), x.DarkSoulCD)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=ticks_remain<=2&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and x.CorruptionTicksRemain <= 2 and x.CorruptionRemains < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(172--[[Corruption]], "spell_power") and (x.TrinketIntProcStack == 0 or x.TrinketIntProcStack > 7) and x.CorruptionTicksRemain * 2 < x.CorruptionTicks and x.CorruptionRemains < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=buff.perfect_aim.react&(crit_pct<100|ticks_remain<=add_ticks)"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not x.Metamorphosis and x.PerfectAimDuration > x.GCD and (a.getDoTStat(172--[[Corruption]], "crit_pct") < 100 or x.CorruptionTicksRemain <= x.CorruptionTicks)
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=!ticking&target.time_to_die>=6&miss_react"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 6 and not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=spell_power<stat.spell_power&ticks_remain<=add_ticks%2&target.time_to_die>=6&miss_react&crit_pct<100"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not x.Metamorphosis and a.getDoTStat(172--[[Corruption]], "spell_power") < GetSpellBonusDamage(6) and x.CorruptionTicksRemain * 2 <= x.CorruptionTicks and x.TimeToDie >= 6 and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100
	end,
	EnemyTargetNeeded = true,
}

a.spells["corruption,cycle_targets=1,if=spell_power<stat.spell_power&ticks_remain<=add_ticks%2&target.time_to_die>=6&miss_react"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not x.Metamorphosis and a.getDoTStat(172--[[Corruption]], "spell_power") < GetSpellBonusDamage(6) and x.CorruptionTicksRemain * 2 <= x.CorruptionTicks and x.TimeToDie >= 6
	end,
	EnemyTargetNeeded = true,
}

a.spells["unstable_affliction,cycle_targets=1,if=remains<gcd+cast_time&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = min((x.MaxGCD + x.GCD + s.CastTime(30108--[[Unstable Affliction]])), x.DarkSoulCD)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["unstable_affliction,cycle_targets=1,if=(remains-cast_time)%(duration%current_ticks)<=2&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and (x.UARemains - s.CastTime(30108--[[Unstable Affliction]])) / x.TickSpeed <= 2 and x.UARemains < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["unstable_affliction,cycle_targets=1,if=stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains<cooldown.dark_soul.remains&miss_react"] = {
	ID = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(30108--[[Unstable Affliction]], "spell_power") and (x.TrinketIntProcStack == 0 or x.TrinketIntProcStack > 7) and x.UATicksRemain * 2 < x.UATicks and x.UARemains < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["shadowburn,if=ember_react&(burning_ember>3.5|mana.pct<=20|buff.dark_soul.up|target.time_to_die<20|buff.havoc.stack>=1|trinket.proc.intellect.react|(trinket.stacking_proc.intellect.react>7&trinket.stacking_proc.intellect.remains>cast_time))"] = {
	ID = 17877--[[Shadowburn]],
	Check = function(z)
		return x.NoCC and (x.BurningEmbers > 3.5 or s.PowerPercent("player", SPELL_POWER_MANA) <= 20 or x.DarkSoulRemains > x.GCD or x.TimeToDie < 20 or s.BuffStack(80240--[[Havoc]], "player") >= 1 or a.getIntProc() > x.GCD or (x.TrinketIntProcStack > 7 and a.getStackingIntProcRemains() > s.CastTime(17877--[[Shadowburn]]) + x.GCD))
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["chaos_bolt,if=ember_react&target.health.pct>20&(buff.havoc.stack=3&buff.havoc.remains>cast_time)"] = {
	ID = 116858--[[Chaos Bolt]],
	ConsumeBuff = 80240--[[Havoc]],
	ConsumeStack = 3,
	ConsumeBuffUnit = "player",
	Check = function(z)
		return x.NoCC and s.HealthPercent("target") > 20 and s.BuffDuration(80240--[[Havoc]], "player") > s.CastTime(116858--[[Chaos Bolt]]) + x.GCD
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["chaos_bolt,if=ember_react&target.health.pct>20&(buff.backdraft.stack<3|level<86|(active_enemies>1&action.incinerate.cast_time<1))&(burning_ember>(4.5-active_enemies)|buff.dark_soul.remains>cast_time|buff.skull_banner.remains>cast_time|(trinket.proc.intellect.react&trinket.proc.intellect.remains>cast_time)|(trinket.stacking_proc.intellect.react>6&trinket.stacking_proc.intellect.remains>cast_time))"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		local cast_time = s.CastTime(116858--[[Chaos Bolt]])
		return x.NoCC and s.HealthPercent("target") > 20 and (s.BuffStack(117828--[[Backdraft]], "player") < 3 or (x.ActiveEnemies > 1 and s.CastTime(29722--[[Incinerate]]) < 1)) and (x.BurningEmbers > (4.5 - x.ActiveEnemies) or x.DarkSoulRemains > cast_time + x.GCD or s.BuffDuration(114206--[[Skull Banner]], "player") > cast_time or (x.IntProc > cast_time + x.GCD) or (x.TrinketIntProcStack > 6 and a.getStackingIntProcRemains() > cast_time + x.GCD))
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["conflagrate,if=charges=2&buff.havoc.stack=0"] = {
	ID = 17962--[[Conflagrate]],
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC and GetSpellCharges(17962--[[Conflagrate]]) == 2 and not s.Buff(80240--[[Havoc]], "player") 
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["conflagrate"] = {
	ID = 17962--[[Conflagrate]],
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["immolate,cycle_targets=1,if=buff.dark_soul.react&dot.immolate.crit_pct<(stat.crit+30)&ticks_remain<add_ticks%2&miss_react"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and x.DarkSoulRemains > x.GCD and a.getDoTStat(348--[[Immolate]], "crit_pct") < (GetSpellCritChance(3) + 30) and x.ImmolateTicksRemain * 2 < x.ImmolateTicks
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["immolate,cycle_targets=1,if=stat.spell_power>spell_power&ticks_remain<add_ticks%2&miss_react&dot.immolate.crit_pct<(stat.crit+30)"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		local spell_power = a.getDoTStat(348--[[Immolate]], "spell_power")
		local crit_pct = a.getDoTStat(348--[[Immolate]], "crit_pct")
		return x.NoCC and GetSpellBonusDamage(3) > spell_power  and x.ImmolateTicksRemain * 2 < x.ImmolateTicks and crit_pct < (GetSpellCritChance(3) + 30)
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["immolate,cycle_targets=1,if=ticks_remain<cast_time&target.time_to_die>=5&miss_react"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and x.ImmolateTicksRemain < s.CastTime(348--[[Immolate]]) + x.GCD and x.TimeToDie >= 5
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["incinerate"] = {
	ID = 29722--[[Incinerate]],
	Check = function(z)
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["drain_soul,interrupt=1,chain=1,if=target.health.pct<=20"] = {
	ID = 1120--[[Drain Soul]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Orange",
}

a.spells["life_tap,if=buff.dark_soul.down&buff.bloodlust.down&mana.pct<10&target.health.pct<=20"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.PowerPercent("player", SPELL_POWER_MANA) < 10 and s.HealthPercent("target") <= 20 and not a.Bloodlust() and not s.Buff(113860--[[Dark Soul: Misery]], "player")
	end,
}

a.spells["life_tap,if=buff.dark_soul.down&buff.bloodlust.down&mana.pct<50"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.PowerPercent("player", SPELL_POWER_MANA) < 50 and not a.Bloodlust() and not s.Buff(113860--[[Dark Soul: Misery]], "player")
	end,
}

a.spells["life_tap,if=target.health.pct<=20"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.HealthPercent("target") <= 20
	end,
}

a.spells["life_tap,moving=1,if=mana.pct<80&mana.pct<target.health.pct"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.Moving("player") and s.PowerPercent("player", SPELL_POWER_MANA) < 80 and s.PowerPercent("player", SPELL_POWER_MANA) < s.HealthPercent("target")
	end,
}

a.spells["life_tap,if=mana.pct<60"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 60
	end,
}

a.spells["malefic_grasp,chain=1,interrupt_if=cooldown.dark_soul.remains=0|target.health.pct<=20"] = {
	ID = 103103--[[Malefic Grasp]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["fel_flame,moving=1"] = {
	ID = 77799--[[Fel Flame]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and s.Moving("player")
	end,
	EnemyTargetNeeded = true,
}

a.spells["life_tap"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	NoStopChannel = true,
}

a.spells["Command Demon - Wrathstorm"] = {
	ID = 119915--[[Wrathstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119915--[[Wrathstorm]], 119898--[[Command Demon]]},
}

a.spells["Command Demon - Felstorm"] = {
	ID = 119914--[[Felstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119914--[[Felstorm]], 119898--[[Command Demon]]},
}

a.spells["wrathguard:wrathstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Wrathstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["felguard:felstorm"] = {
	Type = "pet",
	ID = 89751--[[Felstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Felstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["metamorphosis,if=(buff.dark_soul.up&buff.dark_soul.remains<demonic_fury%32)|demonic_fury>=950|demonic_fury%32>target.time_to_die|buff.perfect_aim.react|(action.hand_of_guldan.in_flight&dot.shadowflame.remains)"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat
	end,
	CheckLast = function(z)
		return (x.DarkSoulRemains > x.GCD and x.DarkSoulRemains * 32 < x.DemonicFury) or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32 or x.PerfectAimDuration > 0 or (s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target") and s.MyDebuff(47960--[[Shadowflame]], "target"))
--		return (x.DarkSoulRemains > x.GCD and x.DemonicFury > x.DarkSoulRemains * 32) or (x.CorruptionRemains < 5 and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100) or x.DoomDuration < x.GCD or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32 or x.PerfectAimDuration > x.GCD
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["metamorphosis,if=(buff.dark_soul.up&buff.dark_soul.remains<demonic_fury%32)|demonic_fury>=950|demonic_fury%32>target.time_to_die|(action.hand_of_guldan.in_flight&dot.shadowflame.remains)"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat
	end,
	CheckLast = function(z)
		return (x.DarkSoulRemains > x.GCD and x.DarkSoulRemains * 32 < x.DemonicFury) or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32 or (s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target") and s.MyDebuff(47960--[[Shadowflame]], "target"))
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["metamorphosis,if=(buff.dark_soul.up&demonic_fury%32>buff.dark_soul.remains)|(dot.corruption.remains<5&dot.corruption.crit_pct<100)|!dot.doom.ticking|demonic_fury>=950|demonic_fury%32>target.time_to_die|buff.perfect_aim.react"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat
	end,
	CheckLast = function(z)
		return (x.DarkSoulRemains > x.GCD and x.DemonicFury > x.DarkSoulRemains * 32) or (x.CorruptionRemains < 5 and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100) or x.DoomDuration < x.GCD or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32 or x.PerfectAimDuration > x.GCD
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["cancel_metamorphosis,if=buff.metamorphosis.up&buff.dark_soul.down&demonic_fury<=650&target.time_to_die>30&(cooldown.metamorphosis.remains<4|demonic_fury<=300)&!(action.hand_of_guldan.in_flight&dot.shadowflame.remains)"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return x.Metamorphosis and x.DarkSoulRemains <= x.GCD and x.DemonicFury <= 650 and x.TimeToDie > 30 and (s.SpellCooldown(103958--[[Metamorphosis]]) < 4 or x.DemonicFury <= 300) and not (s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target") and s.MyDebuff(47960--[[Shadowflame]], "target"))
	end,
	FlashColor = "Yellow",	
	NoGCD = true,
}

a.spells["cancel_metamorphosis,if=buff.metamorphosis.up&buff.dark_soul.down&demonic_fury<=650&target.time_to_die>30"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return x.Metamorphosis and x.DarkSoulRemains <= x.GCD and x.DemonicFury <= 650 and x.TimeToDie > 30
	end,
	FlashColor = "Yellow",	
	NoGCD = true,
}

a.spells["hand_of_guldan,if=buff.perfect_aim.react&buff.perfect_aim.remains>travel_time"] = {
	ID = 105174--[[Hand of Gul'dan]],
	Check = function(z)
		local travel_time = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 1.5)
		return x.NoCC and not x.Metamorphosis and x.PerfectAimDuration > x.GCD and x.PerfectAimDuration > travel_time
	end,
	EnemyTargetNeeded = true,
}

a.spells["hand_of_guldan,if=!in_flight&dot.shadowflame.remains<travel_time+action.shadow_bolt.cast_time&(charges=2|dot.shadowflame.remains>travel_time|(charges=1&recharge_time<4))"] = {
	ID = 105174--[[Hand of Gul'dan]],
--	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		local travel_time = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 1.5)
		local shadowflame_remains = s.MyDebuffDuration(47960--[[Shadowflame]], "target")
		local charges, maxCharges, timeLastCast, cooldownDuration = GetSpellCharges(105174--[[Hand of Gul'dan]])
		local recharge_time
		if charges < maxCharges then
			recharge_time = cooldownDuration - GetTime() + timeLastCast
		else
			recharge_time = 0
		end
		local in_flight = s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target")
		return x.NoCC and not x.Metamorphosis and not in_flight and shadowflame_remains < (travel_time + x.ShadowBoltCastTime) and (charges == 2 or shadowflame_remains > travel_time or (charges == 1 and recharge_time < 4))
	end,
	EnemyTargetNeeded = true,
}

a.spells["soul_fire,if=buff.metamorphosis.up&buff.molten_core.react&(buff.perfect_aim.react&buff.perfect_aim.remains>cast_time)"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and x.Metamorphosis and x.PerfectAimDuration > s.CastTime(6353--[[Soul Fire]]) + x.GCD
	end,
	EnemyTargetNeeded = true,
	NotWhileMoving = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["soul_fire,if=buff.metamorphosis.up&buff.molten_core.react&(buff.dark_soul.remains<action.shadow_bolt.cast_time|buff.dark_soul.remains>cast_time)"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and x.Metamorphosis and x.SoulFireCondition
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["soul_fire,if=buff.molten_core.react&(buff.dark_soul.remains<action.shadow_bolt.cast_time|buff.dark_soul.remains>cast_time)&(buff.molten_core.react>9|target.health.pct<=28)"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and not x.Metamorphosis and x.SoulFireCondition and (s.BuffStack(122355--[[Molten Core]], "player") >9 or s.HealthPercent("target") <= 28)
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["soul_fire,if=buff.molten_core.react&(buff.dark_soul.remains<action.shadow_bolt.cast_time|buff.dark_soul.remains>cast_time)"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and not x.Metamorphosis and x.SoulFireCondition
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["doom,cycle_targets=1,if=buff.metamorphosis.up&buff.perfect_aim.react&(crit_pct<100|ticks_remain<=add_ticks)"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	FlashNoMacros = false,
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and x.PerfectAimDuration > x.GCD and (a.getDoTStat(603--[[Doom]], "crit_pct") < 100 or x.DoomTicksRemain <= x.DoomTicks)
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["doom,cycle_targets=1,if=buff.metamorphosis.up&(ticks_remain<=1|(ticks_remain+1<n_ticks&buff.dark_soul.up))&target.time_to_die>=30&miss_react&dot.doom.crit_pct<100"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	FlashNoMacros = false,
	Check = function(z)
		return x.NoCC and (x.DoomTicksRemain <= 1 or (x.DoomTicksRemain + 1 < a.getTotalTicks(4) and x.DarkSoulRemains > x.GCD)) and x.TimeToDie >= 30 and a.getDoTStat(603--[[Doom]], "crit_pct") < 100
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["doom,cycle_targets=1,if=buff.metamorphosis.up&(ticks_remain<=1|(ticks_remain+1<n_ticks&buff.dark_soul.up)|(ticks_remain<=add_ticks%2&stat.spell_power>spell_power))&target.time_to_die>=30&miss_react"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	FlashNoMacros = false,
	Check = function(z)
		return x.NoCC and (x.DoomTicksRemain <= 1 or (x.DoomTicksRemain + 1 < a.getTotalTicks(4) and x.DarkSoulRemains > x.GCD) or (x.DoomTicksRemain * 2 <= a.getTotalTicks(4) and GetSpellBonusDamage(6) > a.getDoTStat(603--[[Doom]], "spell_power"))) and x.TimeToDie >= 30
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["touch_of_chaos,if=buff.metamorphosis.up"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["touch_of_chaos,cycle_targets=1,if=buff.metamorphosis.up&dot.corruption.ticking&dot.corruption.remains<1.5"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemains > x.GCD and x.CorruptionRemains < 1.5 + x.GCD
	end,
	EnemyTargetNeeded = true,
	NotWhileMoving = false,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["touch_of_chaos,cycle_targets=1,if=buff.metamorphosis.up&dot.corruption.ticking&dot.corruption.remains<20&dot.corruption.crit_pct<100"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemains > x.GCD and x.CorruptionRemains < 20 + x.GCD and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100
	end,
	EnemyTargetNeeded = true,
	NotWhileMoving = false,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["shadow_bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		return x.NoCC
	end,
	CheckLast = function(z)
		return not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
	FlashID = { 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["flask,type=warm_sun"] = {
	Type = "item",
	ID = 76085--[[Flask of the Warm Sun]],
	Buff = 105691--[[Flask of the Warm Sun]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Green",
}

a.spells["mogu_fish_stew"] = {
	Type = "item",
	ID = 74650--[[Mogu Fish Stew]],
	Buff = 104277--[[Well Fed]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	Check = function(z)
		return not x.InCombat and x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Green",
}

a.spells["jade_serpent_potion"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = 105702--[[Potion of the Jade Serpent]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return not x.InCombat and x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Green",
}

a.spells["jade_serpent_potion,if=buff.bloodlust.react|target.health.pct<=20"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = 105702--[[Potion of the Jade Serpent]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return x.InCombat and x.ShouldTemporaryBuff and x.ActiveEnemy and s.Boss()
	end,
	CheckLast = function(z)
		return a.Bloodlust() or (s.HealthPercent("target") <= 20)
	end,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Carrion Swarm"] = {
	ID = 103967--[[Carrion Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	Interrupt = true,
	FlashColor = "Aqua",
	EnemyTargetNeeded = true,
}

a.spells["Banish"] = {
	ID = 710--[[Banish]],
	EnemyTargetNeeded = 1,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	NotWhileMoving = true,
	FlashColor = "Pink",
}

a.spells["Create Healthstone"] = {
	ID = 6201--[[Create Healthstone]],
	Check = function(z)
		return x.ShouldTemporaryBuff and GetItemCount(5512--[[Healthstone]], true) == 0 and not s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Create Soulwell"] = {
	ID = 29893--[[Create Soulwell]],
	Check = function(z)
		return x.ShouldTemporaryBuff  and GetItemCount(5512--[[Healthstone]], true) == 0 and s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Soulshatter"] = {
	ID = 29858--[[Soulshatter]],
	EnemyTargetNeeded = true,
	FlashColor = "Red",
	Check = function(z)
		return x.ThreatPercent >= 80 and s.InGroup()
	end,
}

a.spells["Demonic Leap"] = {
	ID = 109151--[[Demonic Leap]],
	FlashColor = "Red",
	Check = function(z)
		return (s.HasGlyph(56247--[[Glyph of Falling Meteor]]) and IsFalling()) or s.Buff(134370--[[Down Draft]], "target")
	end,
}

a.spells["Curse of Exhaustion"] = {
	ID = 18223--[[Curse of Exhaustion]],
	Check = function(z)
		return not s.Boss("target") and s.Moving("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	EnemyTargetNeeded = true,
}

a.spells["Dark Regeneration"] = {
	ID = 108359--[[Dark Regeneration]],
	Check = function(z)
		return s.HasTalent(108359--[[Dark Regeneration]]) and s.HealthPercent("player") < 70
	end,
	NoGCD = true,
	FlashColor = "Red",
}

a.spells["Burning Rush"] = {
	ID = 111400--[[Burning Rush]],
	NoRangeCheck = true,
	FlashBlink = true,
	FlashColor = "Aqua",
	Check = function(z)
		local _, runSpeed = GetUnitSpeed("player")
		return s.HasTalent(111400--[[Burning Rush]]) and (runSpeed / 7 * 100 < (100 - 15 * s.DebuffStack(119050--[[Kil'jaeden's Cunning]], "player")) or s.Buff(134370--[[Down Draft]], "target") or s.Buff(123461--[[Get Away!]], "target"))
	end,
}

a.spells["Unbound Will"] = {
	ID = 108482--[[Unbound Will]],
	Check = function(z)
		if not s.HasTalent(108482--[[Unbound Will]]) then
			return false
		end
		local eventIndex = C_LossOfControl.GetNumEvents()
		while (eventIndex > 0) do
			local _, _, _, _, _, timeRemaining = C_LossOfControl.GetEventInfo(eventIndex)
			if timeRemaining > 1.5 then
				return true
			end
			eventIndex = eventIndex - 1
		end
		return s.Debuff(nil, "player", nil, nil, 1)
	end,
	NoGCD = true,
	NoRangeCheck = true,
	FlashBlink = true,
	FlashColor = "Aqua",
}

a.spells["berserking"] = {
	ID = 26297--[[Berserking]],
	Buff = 26297--[[Berserking]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["blood_fury"] = {
	ID = 33702--[[Blood Fury]],
	Buff = 33702--[[Blood Fury]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["lifeblood"] = {
	ID = 121279--[[Lifeblood]],
	Buff = 121279--[[Lifeblood]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["rain_of_fire,if=!ticking&!in_flight&active_enemies>1"] = {
	ID = 104232--[[Rain of Fire]],
	Buff = 104232--[[Rain of Fire]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and x.ActiveEnemies > 1
	end,
}

a.spells["rain_of_fire,if=!ticking&!in_flight,moving=1"] = {
	ID = 104232--[[Rain of Fire]],
	Buff = 104232--[[Rain of Fire]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and s.Moving("player")
	end,
}

a.spells["havoc,target=2,if=active_enemies>1"] = {
	ID = 80240--[[Havoc]],
	Buff = 80240--[[Havoc]],
	BuffUnit = "player",
	StackGiven = 3,
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and x.ActiveEnemies > 1
	end,
	Continue = true,
	FlashColor = "Purple",
}